//*****************************************************************************
//-----------------------------------------------------------------------------
// Fast simulation and reconstruction package for TOP counter (F77 core)
// M. Staric, Sept-2010
//-----------------------------------------------------------------------------
//
// TOPparticle.cc
// C++ interface to F77 functions: MC particle
//-----------------------------------------------------------------------------
//*****************************************************************************

#include <iostream>
#include <iomanip>
#include <math.h>
#include <stdlib.h>
#include "TOPparticle.h"
#include "TOPf77fun.h"

TOPparticle::TOPparticle()
{
  m_X=0;
  m_Y=0;
  m_Z=0;
  m_Px=0;
  m_Py=0;
  m_Pz=0;
  m_Tof=0;
  m_LUND=0;
  m_Q=0;
  m_mass=0;
  m_p=0;
  m_REF=0;
}

TOPparticle::TOPparticle(double x, double y, double z, 
			 double Px, double Py, double Pz,
			 double Tof, int Lund, int label)
{
  initMem(x, y, z, Px, Py, Pz, Tof, Lund, label);
}

TOPparticle::TOPparticle(RandomParticle p)
{
  initMem(p.X(), p.Y(), p.Z(), 
	  p.Px(), p.Py(), p.Pz(), 
	  p.Tstart(), p.Lund(), p.Label() );
}

void TOPparticle::initMem(double x, double y, double z, 
		       double Px, double Py, double Pz,
		       double Tof, int Lund, int label)
{
  m_X=x;
  m_Y=y;
  m_Z=z;
  m_Px=Px;
  m_Py=Py;
  m_Pz=Pz;
  m_Tof=Tof;
  m_LUND=Lund;
  m_Q=(abs(m_LUND)>100?1:-1)*(m_LUND>0?1:-1);
  int i=Hyp();
  double Masses[6]={0, 0.511E-3, 0.10566, 0.13957, 0.49368, 0.93827};
  m_mass=Masses[i];
  m_p=sqrt(m_Px*m_Px+m_Py*m_Py+m_Pz*m_Pz);
  m_REF=label;
}

int TOPparticle::Hyp()
{
  int lundc[5]={11,13,211,321,2212};
  for(int i=0; i<5; i++){
    if(abs(m_LUND)==lundc[i]) return i+1;
  }	
  return 0;
}

void TOPparticle::Dump()
{
  double pi=4*atan(1);
  using namespace std;
  cout << "TOPparticle::Dump(): Ref="<< m_REF <<" Lund="<< m_LUND;
  cout << " charge="<< m_Q<<endl;
  cout <<"  p="<< setprecision(3) << p() <<" GeV/c";
  cout <<"  theta="<< setprecision(3) << theta()/pi*180;
  cout <<"  phi="<< setprecision(3) << phi()/pi*180<<endl;
  cout <<"  x=" << m_X << " cm";
  cout <<"  y=" << m_Y << " cm";
  cout <<"  z=" << m_Z << " cm";
  cout <<"  rho=" << Rho() << " cm\n";
  cout <<"  Tof="<< setprecision(4) << m_Tof << " ns";
  cout <<"  Tlen="<< setprecision(4) << Tlen() << " cm";
  cout <<"  beta="<< setprecision(4) << beta();
  cout << endl;
}

///////////////////////////////////////////////////////////////
// F77 helix functions (see comments in srcF77/top_helix.F)
// the following are arrays:  r0[3], dir[3], hlx[10], hlxl[10]
// variable int* ok is bool (F77: logical)
///////////////////////////////////////////////////////////////

extern "C" {
  void set_helix_(float* r0, float* dir, float* p, float* charge, float* hlx);
  void move_pivot_(float* hlx, float* t);
  void get_hlx_rdir_(float* hlx, float* t, float* r0, float* dir);
  void xsec_plane_x_(float* hlx, float* x0, float* t1, float* t2, int* ok);
  void xsec_plane_y_(float* hlx, float* y0, float* t1, float* t2, int* ok);
  void xsec_plane_z_(float* hlx, float* z0, float* t, int* ok);
  void xsec_cyl_(float* hlx, float* r, float* t1, float* t2, int* ok);
  void hlx_to_local_(float* hlx, int* m, float* hlxl);
}

// move particle along helix for distance Dist

void TOPparticle::swim(double Dist)
{
  float r[3]={m_X, m_Y, m_Z};
  float dir[3]={m_Px/m_p, m_Py/m_p, m_Pz/m_p}; 
  float helix[10];
  float p=m_p; float q=m_Q;
  set_helix_(r, dir, &p,  &q, helix);

  double c0=29.9798458; // speed of light [cm/ns]
  float t=Dist/c0;
  get_hlx_rdir_(helix, &t, r, dir);

  m_X=r[0]; m_Y=r[1]; m_Z=r[2];
  m_Px=dir[0]*m_p; m_Py=dir[1]*m_p; m_Pz=dir[2]*m_p;
  m_Tof+=t/beta();
}

// move particle to surface of cylinder with radius R
// if inside cylinder move forward, else move backward
// if failed, returns false 

bool TOPparticle::toCylinder(double R)
{
  if(Rho()==R) return true;
  
  float r[3]={m_X, m_Y, m_Z};
  float dir[3]={m_Px/m_p, m_Py/m_p, m_Pz/m_p}; 
  float helix[10];
  float p=m_p; float q=m_Q;
  set_helix_(r, dir, &p,  &q, helix);

  float rad=R; 
  float t1, t2; 
  int ok;
  xsec_cyl_(helix, &rad, &t1, &t2, &ok);
  if(ok==0) return false;

  float t=Rho()<R ? t1 : t2-helix[9];

  get_hlx_rdir_(helix, &t, r, dir);
  m_X=r[0]; m_Y=r[1]; m_Z=r[2];
  m_Px=dir[0]*m_p; m_Py=dir[1]*m_p; m_Pz=dir[2]*m_p;
  m_Tof+=t/beta();  

  return true;
} 

